@@ -7,19 +7,20 @@
     :license: BSD, see LICENSE for details.
 """
 
-from __future__ import print_function
-
 import os
 import pprint
 import difflib
 import pickle
 
+import pytest
+
 from pygments.lexers import get_lexer_for_filename, get_lexer_by_name
 from pygments.token import Error
 from pygments.util import ClassNotFound
 
-import support
-
+# You can set this to True to store the exact token type output of example
+# files in tests/examplefiles/output, and on the next run the test will
+# want them to stay the same.  In the repository, this should stay False.
 STORE_OUTPUT = False
 
 STATS = {}
@@ -32,9 +33,11 @@ TESTDIR = os.path.dirname(__file__)
 BAD_FILES_FOR_JYTHON = ('Object.st', 'all.nit', 'genclass.clj',
                         'ragel-cpp_rlscan')
 
-def test_example_files():
-    global STATS
-    STATS = {}
+
+def get_example_files():
+    # TODO: move stats to a fixture
+    # global STATS
+    # STATS = {}
     outdir = os.path.join(TESTDIR, 'examplefiles', 'output')
     if STORE_OUTPUT and not os.path.isdir(outdir):
         os.makedirs(outdir)
@@ -51,48 +54,49 @@ def test_example_files():
             continue
 
         print(absfn)
-        with open(absfn, 'rb') as f:
-            code = f.read()
+        yield fn
+
+    # N = 7
+    # stats = list(STATS.items())
+    # stats.sort(key=lambda x: x[1][1])
+    # print('\nExample files that took longest absolute time:')
+    # for fn, t in stats[-N:]:
+    #     print('%-30s  %6d chars  %8.2f ms  %7.3f ms/char' % ((fn,) + t))
+    # print()
+    # stats.sort(key=lambda x: x[1][2])
+    # print('\nExample files that took longest relative time:')
+    # for fn, t in stats[-N:]:
+    #     print('%-30s  %6d chars  %8.2f ms  %7.3f ms/char' % ((fn,) + t))
+
+
+@pytest.mark.parametrize('filename', get_example_files())
+def test_examplefile(filename):
+    if os.name == 'java' and filename in BAD_FILES_FOR_JYTHON:
+        pytest.skip('%s is a known bad file on Jython' % filename)
+
+    absfn = os.path.join(TESTDIR, 'examplefiles', filename)
+    with open(absfn, 'rb') as f:
+        text = f.read()
+    try:
+        utext = text.decode('utf-8')
+    except UnicodeError:
+        utext = text.decode('latin1')
+
+    lx = None
+    if '_' in filename:
         try:
-            code = code.decode('utf-8')
-        except UnicodeError:
-            code = code.decode('latin1')
+            lx = get_lexer_by_name(filename.split('_')[0])
+        except ClassNotFound:
+            pass
+    if lx is None:
+        try:
+            lx = get_lexer_for_filename(absfn, code=utext)
+        except ClassNotFound:
+            raise AssertionError('file %r has no registered extension, '
+                                 'nor is of the form <lexer>_filename '
+                                 'for overriding, thus no lexer found.'
+                                 % filename)
 
-        lx = None
-        if '_' in fn:
-            try:
-                lx = get_lexer_by_name(fn.split('_')[0])
-            except ClassNotFound:
-                pass
-        if lx is None:
-            try:
-                lx = get_lexer_for_filename(absfn, code=code)
-            except ClassNotFound:
-                raise AssertionError('file %r has no registered extension, '
-                                     'nor is of the form <lexer>_filename '
-                                     'for overriding, thus no lexer found.'
-                                     % fn)
-        yield check_lexer, lx, fn
-
-    N = 7
-    stats = list(STATS.items())
-    stats.sort(key=lambda x: x[1][1])
-    print('\nExample files that took longest absolute time:')
-    for fn, t in stats[-N:]:
-        print('%-30s  %6d chars  %8.2f ms  %7.3f ms/char' % ((fn,) + t))
-    print()
-    stats.sort(key=lambda x: x[1][2])
-    print('\nExample files that took longest relative time:')
-    for fn, t in stats[-N:]:
-        print('%-30s  %6d chars  %8.2f ms  %7.3f ms/char' % ((fn,) + t))
-
-
-def check_lexer(lx, fn):
-    if os.name == 'java' and fn in BAD_FILES_FOR_JYTHON:
-        raise support.SkipTest('%s is a known bad file on Jython' % fn)
-    absfn = os.path.join(TESTDIR, 'examplefiles', fn)
-    with open(absfn, 'rb') as fp:
-        text = fp.read()
     text = text.replace(b'\r\n', b'\n')
     text = text.strip(b'\n') + b'\n'
     try:
@@ -122,7 +126,7 @@ def check_lexer(lx, fn):
     # check output against previous run if enabled
     if STORE_OUTPUT:
         # no previous output -- store it
-        outfn = os.path.join(TESTDIR, 'examplefiles', 'output', fn)
+        outfn = os.path.join(TESTDIR, 'examplefiles', 'output', filename)
         if not os.path.isfile(outfn):
             with open(outfn, 'wb') as fp:
                 pickle.dump(tokens, fp)
